/*!
@file
Defines `boost::hana::fold`.

Copyright Louis Dionne 2013-2022
Distributed under the Boost Software License, Version 1.0.
(See accompanying file LICENSE.md or copy at http://boost.org/LICENSE_1_0.txt)
 */

#ifndef BOOST_HANA_FOLD_HPP
#define BOOST_HANA_FOLD_HPP

#include <boost/hana/fwd/fold.hpp>

#include <boost/hana/fold_left.hpp>

#endif // !BOOST_HANA_FOLD_HPP
