//=======================================================================
// Copyright 2005 Trustees of Indiana University
// Authors: Andrew Lumsdaine, Douglas Gregor
//
// Distributed under the Boost Software License, Version 1.0. (See
// accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
//=======================================================================
#ifndef BOOST_GRAPH_SIMPLE_POINT_HPP
#define BOOST_GRAPH_SIMPLE_POINT_HPP

namespace boost
{

template < typename T > struct simple_point
{
    T x;
    T y;
};

} // end namespace boost

#endif // BOOST_GRAPH_SIMPLE_POINT_HPP
